@@ -182,7 +182,7 @@ __glXFreeGLXWindow(__glXWindow * pGlxWindow)
         WindowPtr pWindow = (WindowPtr) pGlxWindow->pDraw;
         WindowPtr ret;
 
-        dixLookupResourceByType((pointer) &ret,
+        dixLookupResourceByType((void *) &ret,
                                 pWindow->drawable.id, RT_WINDOW,
                                 NullClient, DixUnknownAccess);
         if (ret == pWindow) {
@@ -323,12 +323,6 @@ GlxExtensionInit(void)
         FatalError("__glXExtensionInit: AddExtensions failed\n");
         return;
     }
-    /*
-       if (!AddExtensionAlias(GLX_EXTENSION_ALIAS, extEntry)) {
-       ErrorF("__glXExtensionInit: AddExtensionAlias failed\n");
-       return;
-       }
-     */
 
     __glXerrorBase = extEntry->errorBase;
     __glXBadContext = extEntry->errorBase + GLXBadContext;
@@ -347,7 +341,7 @@ GlxExtensionInit(void)
     /*
      ** Initialize table of client state.  There is never a client 0.
      */
-    for (i = 1; i <= MAXCLIENTS; i++) {
+    for (i = 1; i <= LimitClients; i++) {
         __glXClients[i] = 0;
     }
 
@@ -414,7 +408,7 @@ __glXDispatch(ClientPtr client)
          */
         XID xid = FakeClientID(client->index);
 
-        if (!AddResource(xid, __glXClientRes, (pointer) (long) client->index)) {
+        if (!AddResource(xid, __glXClientRes, (void *) (long) client->index)) {
             return BadAlloc;
         }
         ResetClientState(client->index);
@@ -468,7 +462,7 @@ __glXSwapDispatch(ClientPtr client)
          */
         XID xid = FakeClientID(client->index);
 
-        if (!AddResource(xid, __glXClientRes, (pointer) (long) client->index)) {
+        if (!AddResource(xid, __glXClientRes, (void *) (long) client->index)) {
             return BadAlloc;
         }
         ResetClientState(client->index);
